use TestRust;
use std::{env, process, thread};
use TestRust::{Config};
use std::time::Duration;
use TestRust::PrimaryColor;
use TestRust::mix;

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn it_adds_two() {
        assert_eq!(4, TestRust::add_two(2));
    }

    #[test]
    fn call_with_different_values() {
        let mut c = Cacher::new(|a| a);

        let v1 = c.value(1);
        let v2 = c.value(2);

        assert_eq!(v2, 2);
    }

    #[test]
    fn iterator_sum() {
      /*  let v1 = vec![1, 2, 3];

        let v1_iter = v1.iter();

        let total: i32 = v1_iter.sum();

        assert_eq!(total, 6);
        v1.iter().map(|x| x + 1);*/

        let v1: Vec<i32> = vec![1, 2, 3];

        let v2: Vec<_> = v1.iter().map(|x| x + 1).collect();

        assert_eq!(v2, vec![2, 3, 4]);
    }
}
fn main() {
   // let args: Vec<String> = env::args().collect();
  //  let config = Config::new(&args);
    let config = Config::new(env::args()).unwrap_or_else(|err| {
      //  println!("Problem parsing arguments: {}", err);
        eprintln!("Problem parsing arguments: {}", err);
        process::exit(1);
    });

    println!("Searching for {}", config.query);
    println!("In file {}", config.filename);
    if let Err(e) = TestRust::run(config) {
       // println!("Application error: {}", e);
        eprintln!("Application error: {}", e);

        process::exit(1);
    }

    let simulated_user_specified_value = 10;
    let simulated_random_number = 7;

    generate_workout(simulated_user_specified_value, simulated_random_number);

   /* let example_closure = |x| x;

    let s = example_closure(String::from("hello"));
    let n = example_closure(5);*/

   /* let x = 4;

    //let equal_to_x = |z| z == x;

    fn equal_to_x(z: i32) -> bool {
        z == x
    }

    let y = 4;

    assert!(equal_to_x(y));*/

    let x = vec![1, 2, 3];

    let equal_to_x = move |z| z == x;

   // println!("can't use x here: {:?}", x);

    let y = vec![1, 2, 3];

    assert!(equal_to_x(y));

    let v1 = vec![1, 2, 3];

    let mut v1_iter = v1.iter();
   /* for val in v1_iter {
        println!("Got: {}", val);
    }*/
    assert_eq!(v1_iter.next(), Some(&1));
    assert_eq!(v1_iter.next(), Some(&2));
    assert_eq!(v1_iter.next(), Some(&3));
    assert_eq!(v1_iter.next(), None);

    let red = PrimaryColor::Red;
    let yellow = PrimaryColor::Yellow;
    mix(red, yellow);
}

fn generate_workout(intensity: u32, random_number: u32) {
   // let expensive_result = simulated_expensive_calculation(intensity);

   /* let expensive_closure = |num| {
        println!("calculating slowly...");
        thread::sleep(Duration::from_secs(2));
        num
    };*/

    let mut expensive_result = Cacher::new(|num| {
        println!("calculating slowly...");
        thread::sleep(Duration::from_secs(2));
        num
    });

    if intensity < 25 {
        // println!("Today, do {} pushups!", expensive_closure(intensity));
        // println!("Next, do {} situps!", expensive_closure(intensity));
        println!("Today, do {} pushups!", expensive_result.value(intensity));
        println!("Next, do {} situps!", expensive_result.value(intensity));
    } else {
        if random_number == 3 {
            println!("Take a break today! Remember to stay hydrated!");
        } else {
          /*  println!(
                "Today, run for {} minutes!",
                expensive_closure(intensity)
            );*/
            println!(
                "Today, run for {} minutes!",
                expensive_result.value(intensity)
            );
        }
    }
}

fn simulated_expensive_calculation(intensity: u32) -> u32 {
    println!("calculating slowly...");
    thread::sleep(Duration::from_secs(2));
    intensity
}


struct Cacher<T>
    where
        T: Fn(u32) -> u32,
{
    calculation: T,
    value: Option<u32>,
}

impl<T> Cacher<T>
    where
        T: Fn(u32) -> u32,
{
    fn new(calculation: T) -> Cacher<T> {
        Cacher {
            calculation,
            value: None,
        }
    }

    fn value(&mut self, arg: u32) -> u32 {
        match self.value {
            Some(v) => v,
            None => {
                let v = (self.calculation)(arg);
                self.value = Some(v);
                v
            }
        }
    }
}
